/*************************************************************************
*   File: class.c                                       Part of CircleMUD *
*  Usage: Source file for class-specific code                             *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "structs.h"
#include "db.h"
#include "utils.h"
#include "spells.h"
#include "interpreter.h"
#include "comm.h"
#include "handler.h"
#include "newmagic.h"
#include "awake.h"

extern bool help_to_buf(char *, char *);
extern void save_etext(struct char_data *ch);
extern void write_spells(struct char_data *ch);
extern char *motd;
extern const char *spells[];
extern const char *status_ratings[];
extern int top_of_zone_table;
extern spell_a grimoire[];
extern spell_t *find_spell(struct char_data *, char *);
extern struct obj_data *obj_proto;
extern struct index_data *obj_index;
extern struct zone_data *zone_table;
extern void add_page_string(struct descriptor_data *d, char *str);
extern void Crash_rentsave(struct char_data * ch, int cost);
extern void obj_to_char(struct obj_data *object, struct char_data *ch);

const char *race_abbrevs[] = {
  "Hu",
  "Dw",
  "El",
  "Or",
  "Tr",
  "\n"
};

const char *pc_archetypes[] = {
  "Mage",
  "Shaman",
  "Decker",
  "Samurai",
  "Adept",
  "\n"
};

const char *pc_race_types[] = {
  "Human",
  "Dwarf",
  "Elf",
  "Ork",
  "Troll",
  "\n"
};

const char *totem_types[] = {
  "UNDEFINED",
  "Bear",
  "Eagle",
  "Raven",
  "Wolf",
  "Shark",
  "Lion",
  "Coyote",
  "Gator",
  "Owl",
  "Snake",
  "Raccoon",
  "Cat",
  "Dog",
  "Rat",
  "Badger",
  "Bat",
  "Boar",
  "Bull",
  "Cheetah",
  "Cobra",
  "Crab",
  "Crocodile",
  "Dove",
  "Elk",
  "Fish",
  "Fox",
  "Gecko",
  "Goose",
  "Horse",
  "Hyena",
  "Jackal",
  "Jaguar",
  "Leopard",
  "Lizard",
  "Monkey",
  "Otter",
  "Parrot",
  "Polecat",
  "Prarie Dog",
  "Puma",
  "Scorpion",
  "Spider",
  "Stag",
  "Turtle",
  "Whale",
  "Moon",
  "Mountain",
  "Oak",
  "Sea",
  "Stream",
  "Sun",
  "Wind",
  "\n"
};

const char *allergy_names[] = {
  "Anything :)",
  "^LSilver^n",
  "^CIron^n",
  "^WPlastic^n",
  "^YSunlight^n",
  "\n"
};

const char *severity_names[] = {
  "Not",
  "Slightly",
  "Mildly",
  "Moderately",
  "Severely",
  "\n"
};

const char *archetype_menu =
"\r\nSelect an archetype:\r\n"
"  [1] Street Samurai\r\n"
"  [2] Shaman\r\n"
"  [3] Mage\r\n"
"  [4] Decker\r\n"
"  [5] Physical Adept\r\n"
"  ?# (for help on a particular archetype)\r\n";

const char *race_menu =
"\r\nSelect a race:\r\n"
"  [1] Human\r\n"
"  [2] Dwarf\r\n"
"  [3] Elf\r\n"
"  [4] Ork\r\n"
"  [5] Troll\r\n"
"  ?# (for help on a particular race)\r\n";

const char *totem_menu =
"\r\nSelect your totem:\r\n"
"    Wilderness:\r\n"
"     [1] Bear          [2] Eagle         [3] Raven\r\n"
"     [4] Wolf          [5] Shark         [6] Lion\r\n\r\n"
"    Urban:\r\n"
"     [7] Dog           [8] Cat           [9] Rat\r\n\r\n"
"    Both Urban and Wilderness:\r\n"
"     [10] Coyote       [11] Gator        [12] Owl\r\n"
"     [13] Snake        [14] Raccoon\r\n"
"  ?# (for help on a particular totem)\r\n";

const char *assign_menu =
"\r\nSelect a priority to assign:\r\n"
"  [1] Attributes\r\n"
"  [2] Magic\r\n"
"  [3] Resources\r\n"
"  [4] Skills\r\n";

const char *equip_menu =
"\r\nSelect an item type to purchase:\r\n"
"  [1] Weapons\r\n"
"  [2] Armor\r\n"
"  [3] Cyberdecks & programs\r\n"
"  [4] Cyberware\r\n"
"  [5] Continue\r\n";

const char *weapons_menu =
"\r\n##  Name                      Weight     Cost\r\n"
"1   Katana                      1        1,000 \r\n"
"2   Knife                       1        30 \r\n"
"3   Sword                       1        500 \r\n"
"4   Spear                       4        500 \r\n"
"5   Stun baton                  1        750 \r\n"
"6   Beretta 101T                1        350 \r\n"
"7   Ares Predator ||            2        550 \r\n"
"8   Browning Max-Power          2        450 \r\n"
"9   Ruger Super Warhawk         3        300 \r\n"
"10  Enfield AS-7                4        1,000 \r\n";

const char *armor_menu =
"\r\n##  Name                      Weight     Cost\r\n"
"1   Armor shirt                 2        400 \r\n"
"2   Armor vest                  1        200 \r\n"
"3   Armor pants                 1        150 \r\n"
"4   Armor jacket                2        900 \r\n";

const char *cyberdeck_menu =
"\r\n##  Name                   MPCP  Hardening  Memory  Storage  Size
Cost\r\n"
"1   Radio Shack PCD-100     2        0        10      50      -    6,800 \r\n"
"2   Allegiance Alpha        3        1        10      50      -    12,600 \r\n"
"3   Bod program      R:2    -        -        -       -       12   1,200 \r\n"
"4   Evasion program  R:2    -        -        -       -       12   1,200 \r\n"
"5   Masking program  R:2    -        -        -       -       8    800 \r\n"
"6   Sensors program  R:2    -        -        -       -       8    800 \r\n"
"7   Attack program   R:2    -        -        -       -       8    800 \r\n";

const char *cyberware_menu =
"\r\n##  Name                      Essence    Cost\r\n"
"1   Datajack                    0.2      1,000 \r\n"
"2   Radio                       0.5      4,000 \r\n"
"3   Radio receiver              0.25     2,000 \r\n"
"4   Commlink                    0.25     3,700 \r\n"
"5   Rectractable hand razors    0.2      9,000 \r\n"
"6   Retractable hand spurs      0.3      11,500 \r\n";

const char *spell_menu =
"\r\n##  Spell\r\n"
"1   Mana dart\r\n"
"2   Mana missile\r\n"
"3   Mana bolt\r\n"
"4   Power dart\r\n"
"5   Power missile\r\n"
"6   Power bolt\r\n"
"7   Heal\r\n";

const char *type_desc =
"\r\nArchetypal settings are standardized, depending on which archetype is\r\n"
"chosen.  Attribute points, skill points, starting equipment, and starting\r\n"
"spells are pre-chosen.  Recommended for inexperienced Shadowrun players.\r\n"
"\r\nCustomized settings offer much more variety and flexibility.  Using this,\r\n"
"players can assign their own priorities, purchase starting equipment, and if\r\n"
"applicable, learn their starting spells and choose their tradition.\r\n";

int attrib_vals[5] = { 20, 15, 13, 11, 9 };
int skill_vals[5] = { 25, 19, 16, 14, 10 };
int nuyen_vals[5] = { 100000, 40000, 9000, 500, 50 };
int force_vals[5] = { 25, 15, 10, 5, 0 };

void set_attributes(struct char_data *ch, int magic)
{
  ch->real_abils.bod = 1;
  ch->real_abils.qui = 1;
  ch->real_abils.str = 1;
  ch->real_abils.cha = 1;
  ch->real_abils.intel = 1;
  ch->real_abils.wil = 1;
  ch->real_abils.ess = 600;
  ch->real_abils.bod_index = 100;
  ch->real_abils.rea = 1;

  if (magic)
  {
    ch->real_abils.mag = 600;
  	ch->real_abils.rmag = 600;
  }
  else ch->real_abils.mag = 0;

  switch (GET_RACE(ch)) {
    case RACE_DWARF:
      ch->real_abils.bod++;
      ch->real_abils.str += 2;
      ch->real_abils.wil++;
      ch->real_abils.bod_index += 100;
      GET_ATT_POINTS(ch)--;
      break;
    case RACE_ELF:
      ch->real_abils.qui++;
      ch->real_abils.cha += 2;
      break;
    case RACE_ORK:
      ch->real_abils.bod += 3;
      ch->real_abils.str += 2;
      ch->real_abils.bod_index += 300;
      GET_ATT_POINTS(ch) -= 2;
      break;
    case RACE_TROLL:
      ch->real_abils.bod += 5;
      ch->real_abils.str += 4;
      ch->real_abils.bod_index += 500;
      GET_ATT_POINTS(ch) -= 6;
      break;
  }

  if (GET_RACE(ch) == RACE_HUMAN) {
    GET_ALLERGY(ch) = ALLERGIC_NONE;
    GET_SEVERITY(ch) = REACT_NONE;
  } else {
    GET_ALLERGY(ch) = number(ALLERGIC_NONE, ALLERGIC_SUNLIGHT);
    if (GET_ALLERGY(ch) == ALLERGIC_NONE)
      GET_SEVERITY(ch) = REACT_NONE;
    else {
      GET_SEVERITY(ch) = number(REACT_NUISANCE, REACT_SEVERE);
      if (GET_SEVERITY(ch) > REACT_MILD) {
        GET_ATT_POINTS(ch) += GET_SEVERITY(ch) - 1;
        GET_SKILL_POINTS(ch) += GET_SEVERITY(ch);
        ch->desc->ccr.force_points += GET_SEVERITY(ch) - 1;
        GET_NUYEN(ch) += (GET_SEVERITY(ch) - 1) * 100;
      } else if (GET_SEVERITY(ch) == REACT_NUISANCE) {
        GET_ATT_POINTS(ch) += (GET_ALLERGY(ch) + 1) >> 1;
        GET_SKILL_POINTS(ch) += (GET_ALLERGY(ch) + 1) >> 1;
        ch->desc->ccr.force_points += (GET_ALLERGY(ch) + 1) >> 1;
        GET_NUYEN(ch) += ((GET_ALLERGY(ch) + 1) >> 1) * 200;
      } else if (GET_ALLERGY(ch) > ALLERGIC_IRON) {
        GET_ATT_POINTS(ch) += 2;
        GET_SKILL_POINTS(ch) += 2;
        ch->desc->ccr.force_points += 2;
        GET_NUYEN(ch) += 400;
      } else {
        GET_ATT_POINTS(ch)++;
        GET_SKILL_POINTS(ch) += 2;
        ch->desc->ccr.force_points++;
        GET_NUYEN(ch) += 100;
      }
    }
  }
  ch->aff_abils = ch->real_abils;
}

/* Some initializations for characters, including initial skills */
void do_start(struct char_data * ch)
{
  void advance_level(struct char_data * ch);
  struct obj_data *obj;
  int r_num;

  GET_LEVEL(ch) = 1;
  GET_KARMA(ch) = 0;
  GET_REP(ch) = 0;

  set_title(ch, "The Newbie");
  set_pretitle(ch, NULL);
/*  set_whotitle(ch, "     "); */
  switch(GET_RACE(ch))
  {
	case RACE_HUMAN:
		set_whotitle(ch, "Human");
		break;
	case RACE_DWARF:
		set_whotitle(ch, "Dwarf");
		break;
	case RACE_ELF:
		set_whotitle(ch, " Elf ");
		break;
	case RACE_ORK:
		set_whotitle(ch, " Ork ");
		break;
	case RACE_TROLL:
		set_whotitle(ch, "Troll");
		break;
	default:
		mudlog("No race found at set_whotitle in class.cc", NULL, LOG_SYSLOG, TRUE);
		set_whotitle(ch, "CHKLG"); /* Will set incase the players */
  }					   /* race is undeterminable      */

  ch->points.max_physical = 1000;
  ch->points.max_mental = 1000;

  ch->char_specials.saved.left_handed = (!number(0, 9) ? 1 : 0);
  GET_WIELDED(ch, 0) = 0;
  GET_WIELDED(ch, 1) = 0;

  advance_level(ch);

  GET_PHYSICAL(ch) = GET_MAX_PHYSICAL(ch);
  GET_MENTAL(ch) = GET_MAX_MENTAL(ch);
  //GET_MOVE(ch) = GET_MAX_MOVE(ch);

  GET_COND(ch, THIRST) = 24;
  GET_COND(ch, FULL) = 24;
  GET_COND(ch, DRUNK) = 0;
  GET_LOADROOM(ch) = 8039;
  GET_PROMPT(ch) = str_dup("< @pP @mM > ");

  SET_BIT(PLR_FLAGS(ch), PLR_NEWBIE);
  SET_BIT(PRF_FLAGS(ch), PRF_AUTOEXIT);

  ch->player.time.played = 0;
  ch->player.time.logon = time(0);

  GET_NUYEN(ch) = 0;
}

/* Gain maximum in various points */
void advance_level(struct char_data * ch)
{
  int i;

  if (GET_LEVEL(ch) >= LVL_LEGEND)
  {
    for (i = 0; i < 3; i++)
      GET_COND(ch, i) = (char) -1;
    GET_KARMA(ch) = 0;
    GET_REP(ch) = 0;
    if (PRF_FLAGGED(ch, PRF_PKER))
      REMOVE_BIT(PLR_FLAGS(ch), PRF_PKER);
    SET_BIT(PRF_FLAGS(ch), PRF_HOLYLIGHT);
    SET_BIT(PRF_FLAGS(ch), PRF_ROOMFLAGS);
    SET_BIT(PRF_FLAGS(ch), PRF_NOHASSLE);
  }

  save_char(ch, NOWHERE);
  write_spells(ch);
  save_etext(ch);

  sprintf(buf, "%s advanced to %s.", GET_NAME(ch), status_ratings[GET_LEVEL(ch)]);
  mudlog(buf, ch, LOG_MISCLOG, TRUE);
}

void init_create_vars(struct descriptor_data *d)
{
  int i;

  d->ccr.mode = CCR_SEX;
  d->ccr.archetype = AT_UNDEFINED;
  for (i = 0; i < 5; i++)
    d->ccr.pr[i] = PR_NONE;
  d->ccr.force_points = 0;
  d->ccr.temp = 0;
}

int parse_archetype(struct descriptor_data *d, char *arg)
{
  switch (*arg) {
    case '1': return AT_SAMURAI;
    case '2': return AT_SHAMAN;
    case '3': return AT_MAGE;
    case '4': return AT_DECKER;
    case '5': return AT_ADEPT;
    case '?':
      switch (*(arg+1)) {
        case '1': help_to_buf(buf2, "street samurai"); break;
        case '2': help_to_buf(buf2, "shaman");         break;
        case '3': help_to_buf(buf2, "mage");           break;
        case '4': help_to_buf(buf2, "decker");         break;
        case '5': help_to_buf(buf2, "adept");          break;
        default:  return AT_UNDEFINED;
      }
      strcat(buf2, "\r\n Press [return] to continue ");
      SEND_TO_Q(buf2, d);
      d->ccr.temp = CCR_ARCHETYPE;
      d->ccr.mode = CCR_AWAIT_CR;
      return RETURN_HELP;
    default:  return AT_UNDEFINED;
  }
}

int parse_race(struct descriptor_data *d, char *arg)
{
  switch (*arg) {
    case '1': return RACE_HUMAN;
    case '2': return RACE_DWARF;
    case '3': return RACE_ELF;
    case '4': return RACE_ORK;
    case '5': return RACE_TROLL;
    case '?':
      switch (*(arg+1)) {
        case '1': help_to_buf(buf2, "human"); break;
        case '2': help_to_buf(buf2, "dwarf"); break;
        case '3': help_to_buf(buf2, "elf");   break;
        case '4': help_to_buf(buf2, "ork");   break;
        case '5': help_to_buf(buf2, "troll"); break;
        default:  return RACE_UNDEFINED;
      }
      strcat(buf2, "\r\n Press [return] to continue ");
      SEND_TO_Q(buf2, d);
      d->ccr.temp = CCR_RACE;
      d->ccr.mode = CCR_AWAIT_CR;
      return RETURN_HELP;
    default:  return RACE_UNDEFINED;
  }
}

int parse_totem(struct descriptor_data *d, char *arg)
{
  char *temp = arg;
  int i;

  if (*temp == '?') {
    i = atoi(++temp);
    switch (i) {
      case 1:  help_to_buf(buf2, "bear");    break;
      case 2:  help_to_buf(buf2, "eagle");   break;
      case 3:  help_to_buf(buf2, "raven");   break;
      case 4:  help_to_buf(buf2, "wolf");    break;
      case 5:  help_to_buf(buf2, "shark");   break;
      case 6:  help_to_buf(buf2, "lion");    break;
      case 7:  help_to_buf(buf2, "dog");     break;
      case 8:  help_to_buf(buf2, "cat");     break;
      case 9:  help_to_buf(buf2, "rat");     break;
      case 10: help_to_buf(buf2, "coyote");  break;
      case 11: help_to_buf(buf2, "gator");   break;
      case 12: help_to_buf(buf2, "owl");     break;
      case 13: help_to_buf(buf2, "snake");   break;
      case 14: help_to_buf(buf2, "raccoon"); break;
      default: return TOTEM_UNDEFINED;
    }
    strcat(buf2, "\r\n Press [return] to continue ");
    SEND_TO_Q(buf2, d);
    d->ccr.temp = CCR_TOTEM;
    d->ccr.mode = CCR_AWAIT_CR;
    return RETURN_HELP;
  }

  i = atoi(arg);
  switch (i) {
    case 1:  return TOTEM_BEAR;
    case 2:  return TOTEM_EAGLE;
    case 3:  return TOTEM_RAVEN;
    case 4:  return TOTEM_WOLF;
    case 5:  return TOTEM_SHARK;
    case 6:  return TOTEM_LION;
    case 7:  return TOTEM_DOG;
    case 8:  return TOTEM_CAT;
    case 9:  return TOTEM_RAT;
    case 10: return TOTEM_COYOTE;
    case 11: return TOTEM_GATOR;
    case 12: return TOTEM_OWL;
    case 13: return TOTEM_SNAKE;
    case 14: return TOTEM_RACCOON;
    default: return TOTEM_UNDEFINED;
  }
}

void assign_priorities(struct descriptor_data *d)
{
  struct obj_data *obj;
  int i = 0;

  if (GET_RACE(d->character) != RACE_HUMAN)
    i++;
  if (GET_TRADITION(d->character) == TRAD_HERMETIC ||
      GET_TRADITION(d->character) == TRAD_SHAMANIC)
    i++;
  GET_SKILL_POINTS(d->character) = skill_vals[i];
  if (GET_TRADITION(d->character) == TRAD_ADEPT)
    i += 2;
  else i++;
  GET_ATT_POINTS(d->character) = attrib_vals[i];
  GET_NUYEN(d->character) = nuyen_vals[i+1];
  d->ccr.force_points = force_vals[i+1];
  // weapon
  if (GET_NUYEN(d->character) > 1000) {
    GET_NUYEN(d->character) -= 1000;
    obj = read_object(631, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  } else if (GET_NUYEN(d->character) > 550) {
    GET_NUYEN(d->character) -= 550;
    obj = read_object(621, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  } else if (GET_NUYEN(d->character) > 350) {
    GET_NUYEN(d->character) -= 350;
    obj = read_object(618, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  } else {
    obj = read_object(652, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  }
  // main armor
  if (GET_NUYEN(d->character) > 900) {
    GET_NUYEN(d->character) -= 900;
    obj = read_object(1913, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  } else if (GET_NUYEN(d->character) > 400) {
    GET_NUYEN(d->character) -= 400;
    obj = read_object(2017, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  } else if (GET_NUYEN(d->character) > 200) {
    GET_NUYEN(d->character) -= 200;
    obj = read_object(2016, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  } else {
    obj = read_object(1100, VIRTUAL);
    obj_to_char(obj, d->character);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
  }
  if (d->ccr.archetype == AT_SAMURAI) {
    if (GET_NUYEN(d->character) > 4000) {
      GET_NUYEN(d->character) -= 4000;
      obj = read_object(371, VIRTUAL);
      obj_to_cyberware(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
    } else if (GET_NUYEN(d->character) > 2000) {
      GET_NUYEN(d->character) -= 2000;
      obj = read_object(372, VIRTUAL);
      obj_to_cyberware(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
    }
  } else if (d->ccr.archetype == AT_DECKER) {
    if (GET_NUYEN(d->character) > 1000) {
      GET_NUYEN(d->character) -= 1000;
      obj = read_object(304, VIRTUAL);
      obj_to_cyberware(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
    }
    if (GET_NUYEN(d->character) > 6800) {
      GET_NUYEN(d->character) -= 6800;
      obj = read_object(1800, VIRTUAL);
      obj_to_char(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      if (GET_NUYEN(d->character) > 1200) {
        GET_NUYEN(d->character) -= 1200;
        obj = read_object(1821, VIRTUAL);
        obj_to_char(obj, d->character);
        SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      }
      if (GET_NUYEN(d->character) > 1200) {
        GET_NUYEN(d->character) -= 1200;
        obj = read_object(1822, VIRTUAL);
        obj_to_char(obj, d->character);
        SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      }
      if (GET_NUYEN(d->character) > 800) {
        GET_NUYEN(d->character) -= 800;
        obj = read_object(1823, VIRTUAL);
        obj_to_char(obj, d->character);
        SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      }
      if (GET_NUYEN(d->character) > 800) {
        GET_NUYEN(d->character) -= 800;
        obj = read_object(1824, VIRTUAL);
        obj_to_char(obj, d->character);
        SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      }
      if (GET_NUYEN(d->character) > 800) {
        GET_NUYEN(d->character) -= 800;
        obj = read_object(1825, VIRTUAL);
        obj_to_char(obj, d->character);
        SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      }
    }
  } else if (d->ccr.archetype == AT_MAGE || d->ccr.archetype == AT_SHAMAN) {
    int force, num;
    spell_t *spell;

    force = (int)(d->ccr.force_points / 3);
    d->ccr.force_points = (d->ccr.force_points % 3);
    num = SPELL_MANA_DART;
    if (force > 0) {
      while (num != -1) {
        spell = new spell_t;
        i = strlen(spells[num]);
        spell->name = new char[i+1];
        strcpy(spell->name, spells[num]);
        spell->physical = grimoire[num].physical;
        spell->category = grimoire[num].category;
        spell->force = (num == SPELL_POWER_DART ? force + d->ccr.force_points : force);
        spell->target = grimoire[num].target;
        spell->drain = grimoire[num].drain;
        spell->damage = grimoire[num].damage;
        spell->type = num;
        spell->effect = SPELL_ELEMENT_NONE;
        if (!d->character->spells) {
          d->character->spells = spell;
          spell->next = NULL;
        } else {
          spell->next = d->character->spells;
          d->character->spells = spell;
        }
        if (num == SPELL_MANA_DART)
          num = SPELL_POWER_DART;
        else if (num == SPELL_POWER_DART)
          num = SPELL_HEAL;
        else num = -1;
      }
    } else if (d->ccr.force_points > 0) {
      spell = new spell_t;
      i = strlen(spells[SPELL_POWER_DART]);
      spell->name = new char[i+1];
      strcpy(spell->name, spells[SPELL_POWER_DART]);
      spell->physical = grimoire[SPELL_POWER_DART].physical;
      spell->category = grimoire[SPELL_POWER_DART].category;
      spell->force = d->ccr.force_points;
      spell->target = grimoire[SPELL_POWER_DART].target;
      spell->drain = grimoire[SPELL_POWER_DART].drain;
      spell->damage = grimoire[SPELL_POWER_DART].damage;
      spell->type = SPELL_POWER_DART;
      spell->effect = SPELL_ELEMENT_NONE;
      if (!d->character->spells) {
        d->character->spells = spell;
        spell->next = NULL;
      } else {
        spell->next = d->character->spells;
        d->character->spells = spell;
      }
    }
  }
  GET_NUYEN(d->character) = 0;
}

int parse_assign(struct descriptor_data *d, char *arg)
{
  int i;

  switch (*arg) {
    case '1': case '2': case '3': case '4':
      for (i = 0; i < 5; i++)
        if (d->ccr.pr[i] == (int)(*arg - '0')) {
          switch (d->ccr.pr[i]) {
            case PR_ATTRIB:
              GET_ATT_POINTS(d->character) = 0;
              break;
            case PR_MAGIC:
              break;
            case PR_RESOURCE:
              GET_NUYEN(d->character) = 0;
              d->ccr.force_points = 0;
              break;
            case PR_SKILL:
              GET_SKILL_POINTS(d->character) = 0;
              break;
          }
          d->ccr.pr[i] = PR_NONE;
        }
      d->ccr.pr[d->ccr.temp] = (int)(*arg - '0');
      switch (d->ccr.pr[d->ccr.temp]) {
        case PR_ATTRIB:
          GET_ATT_POINTS(d->character) = attrib_vals[d->ccr.temp];
          break;
        case PR_MAGIC:
          break;
        case PR_RESOURCE:
          GET_NUYEN(d->character) = nuyen_vals[d->ccr.temp];
          d->ccr.force_points = force_vals[d->ccr.temp];
          break;
        case PR_SKILL:
          GET_SKILL_POINTS(d->character) = skill_vals[d->ccr.temp];
          break;
      }
      d->ccr.temp = 0;
      return 1;
    case 'c':
      d->ccr.pr[d->ccr.temp] = PR_NONE;
      d->ccr.temp = 0;
      return 1;
  }
  return 0;
}

void priority_menu(struct descriptor_data *d)
{
  int i;

  d->ccr.mode = CCR_PRIORITY;
  SEND_TO_Q("\r\nPriority  Setting     Attributes   Skills Resources\r\n", d);

  for (i = 0; i < 5; i++) {
    sprintf(buf2, "%-10c", 'A' + i);
    switch (d->ccr.pr[i]) {
      case PR_NONE:
        sprintf(buf2, "%s?           %-2d           %-2d        %d  / %d\r\n",
                buf2, attrib_vals[i], skill_vals[i], nuyen_vals[i], force_vals[i]);
        break;
      case PR_RACE:
        strcat(buf2, "Race        -            -         -\r\n");
        break;
      case PR_MAGIC:
        strcat(buf2, "Magic       -            -         -\r\n");
        break;
      case PR_ATTRIB:
        sprintf(buf2, "%sAttributes  %-2d           -         -\r\n", buf2,
                attrib_vals[i]);
        break;
      case PR_SKILL:
        sprintf(buf2, "%sSkills      -            %-2d        -\r\n", buf2,
                skill_vals[i]);
        break;
      case PR_RESOURCE:
        sprintf(buf2, "%sResources   -            -         %d  / %d\r\n",
                buf2, nuyen_vals[i], force_vals[i]);
        break;
    }
    SEND_TO_Q(buf2, d);
  }
  if (d->ccr.pr[0] != PR_RACE)
    SEND_TO_Q("\r\nChoose a priority (A-D) to set (? for help, p to continue): ", d);
  else SEND_TO_Q("\r\nChoose a priority (B-E) to set (? for help, p to continue): ", d);
}

void complete_menu(struct descriptor_data *d)
{
  struct obj_data *obj;
  spell_t *spell;
  int i, num, first = 1;

  if (d->ccr.mode < CCR_SPELLS) {
    *buf2 = '\0';
    if (d->ccr.mode != CCR_CYBERWARE)
      obj = d->character->carrying;
    else obj = d->character->cyberware;
    for (; obj; obj = obj->next_content) {
      num = 1;
      while (obj->next_content) {
        if (obj->item_number != obj->next_content->item_number ||
             strcmp(obj->description, obj->next_content->description))
          break;
        num++;
        obj = obj->next_content;
      }
      switch (d->ccr.mode) {
        case CCR_WEAPON:
          if (GET_OBJ_TYPE(obj) == ITEM_WEAPON) {
            if (first)
              strcat(buf2, "\r\nCurrent weapon(s):\r\n");
            if (num > 1)
              sprintf(buf2, "%s(%d) %s\r\n", buf2, num, obj->short_description);
            else sprintf(buf2, "%s%s\r\n", buf2, obj->short_description);
            first = 0;
          }
          break;
        case CCR_ARMOR:
          if (GET_OBJ_TYPE(obj) == ITEM_ARMOR) {
            if (first)
              strcat(buf2, "\r\nCurrent armor:\r\n");
            if (num > 1)
              sprintf(buf2, "%s(%d) %s\r\n", buf2, num, obj->short_description);
            else sprintf(buf2, "%s%s\r\n", buf2, obj->short_description);
            first = 0;
          }
          break;
        case CCR_CYBERDECK:
          if (GET_OBJ_TYPE(obj) == ITEM_CYBERDECK || GET_OBJ_TYPE(obj) == ITEM_PROGRAM) {
            if (first)
              strcat(buf2, "\r\nCurrent cyberdeck/accessories:\r\n");
            if (num > 1)
              sprintf(buf2, "%s(%d) %s\r\n", buf2, num, obj->short_description);
            else sprintf(buf2, "%s%s\r\n", buf2, obj->short_description);
            first = 0;
          }
          break;
        case CCR_CYBERWARE:
          if (GET_OBJ_TYPE(obj) == ITEM_CYBERWARE) {
            if (first)
              strcat(buf2, "\r\nCurrent cyberware:\r\n");
            sprintf(buf2, "%s%s\r\n", buf2, obj->short_description);
            first = 0;
          }
          break;
      }
    }
    sprintf(buf2, "%s\r\nYou have %d  to spend (remember: all leftover nuyen will "
            "be erased!)\r\nChoose an item to purchase (q to return): ", buf2,
            GET_NUYEN(d->character));
  } else {
    if (d->character->spells)
      strcpy(buf2, "\r\nCurrent spells:\r\n");
    else *buf2 = '\0';
    for (spell = d->character->spells; spell; spell = spell->next)
      sprintf(buf2, "%s%s (force: %d)\r\n", buf2, spell->name, spell->force);
    sprintf(buf2, "%s\r\nYou have %d force points to spend (all leftover points will "
            "be erased!)\r\nChoose a spell to learn (q to continue): ", buf2,
            d->ccr.force_points);
  }

  SEND_TO_Q(buf2, d);
}

void create_parse(struct descriptor_data *d, char *arg)
{
  int i, nuyen, num, rnum, ess, force, ok;
  struct obj_data *obj;
  spell_t *spell, *temp;

  switch (d->ccr.mode) {
    case CCR_AWAIT_CR:
      d->ccr.mode = d->ccr.temp;
      d->ccr.temp = 0;
      switch (d->ccr.mode) {
        case CCR_RACE:
          SEND_TO_Q(race_menu, d);
          SEND_TO_Q("\r\nRace: ", d);
          break;
        case CCR_ARCHETYPE:
          SEND_TO_Q(archetype_menu, d);
          SEND_TO_Q("\r\nArchetype: ", d);
          break;
        case CCR_TOTEM:
          SEND_TO_Q(totem_menu, d);
          SEND_TO_Q("\r\nTotem: ", d);
          break;
        case CCR_PRIORITY:
          priority_menu(d);
          break;
        default:
          log("Invalid mode in AWAIT_CR in chargen.");
      }
      break;
    case CCR_SEX:
      switch (*arg) {
        case 'm': case 'M':
          d->character->player.sex = SEX_MALE;
          break;
        case 'f': case 'F':
          d->character->player.sex = SEX_FEMALE;
          break;
        default:
          SEND_TO_Q("That is not a sex..\r\nWhat IS your sex? ", d);
          return;
      }
      SEND_TO_Q(race_menu, d);
      SEND_TO_Q("\r\nRace: ", d);
      d->ccr.mode = CCR_RACE;
      break;
    case CCR_RACE:
      if ((GET_RACE(d->character) = parse_race(d, arg)) == RACE_UNDEFINED) {
        SEND_TO_Q("\r\nThat's not a race.\r\nRace: ", d);
        return;
      } else if (GET_RACE(d->character) == RETURN_HELP) // for when they use help
        return;
      if (GET_RACE(d->character) == RACE_HUMAN)
        d->ccr.pr[4] = PR_RACE;
      else d->ccr.pr[0] = PR_RACE;
      SEND_TO_Q(type_desc, d);
      SEND_TO_Q("\r\nUse [a]rchetypal or [c]ustomized settings? ", d);
      // all characters get these
      if (real_object(399) > -1)
        obj_to_char(read_object(399, VIRTUAL), d->character);
      if (real_object(2041) > -1)
        obj_to_char(read_object(2041, VIRTUAL), d->character);
      d->ccr.mode = CCR_TYPE;
      break;
    case CCR_TYPE:
      if (*arg == 'a') {
        SEND_TO_Q(archetype_menu, d);
        SEND_TO_Q("\r\nArchetype: ", d);
        d->ccr.mode = CCR_ARCHETYPE;
      } else if (*arg == 'c') {
        priority_menu(d);
        d->ccr.mode = CCR_PRIORITY;
      } else {
        SEND_TO_Q("\r\nThat's not a valid option.\r\n"
                  "Use [a]rchetypal or [c]ustomized settings? ", d);
        return;
      }
      break;
    case CCR_ARCHETYPE:
      if ((d->ccr.archetype = parse_archetype(d, arg)) == AT_UNDEFINED) {
        SEND_TO_Q("\r\nThat's not an archetype.\r\nArchetype: ", d);
        return;
      } else if (d->ccr.archetype == RETURN_HELP) // for when they use help
        return;
      if (d->ccr.archetype == AT_SHAMAN) {
        GET_TRADITION(d->character) = TRAD_SHAMANIC;
        SEND_TO_Q(totem_menu, d);
        SEND_TO_Q("\r\nTotem: ", d);
        set_attributes(d->character, 1);
        d->ccr.mode = CCR_TOTEM;
        return;
      } else if (d->ccr.archetype == AT_MAGE) {
        assign_priorities(d);
        set_attributes(d->character, 1);
        GET_TRADITION(d->character) = TRAD_HERMETIC;
      } else if (d->ccr.archetype == AT_ADEPT) {
        GET_TRADITION(d->character) = TRAD_ADEPT;
        assign_priorities(d);
        set_attributes(d->character, 1);
      } else {
        GET_TRADITION(d->character) = TRAD_MUNDANE;
        assign_priorities(d);
        set_attributes(d->character, 0);
      }
      if (GET_PFILEPOS(d->character) < 0)
        GET_PFILEPOS(d->character) = create_entry(GET_NAME(d->character));
      init_char(d->character);
      d->character->player.host = str_dup(d->host);
      save_char(d->character, NOWHERE);
      Crash_rentsave(d->character, 0);
      write_spells(d->character);
      SEND_TO_Q(motd, d);
      SEND_TO_Q("\r\n\n*** PRESS RETURN: ", d);
      STATE(d) = CON_RMOTD;
      init_create_vars(d);
      sprintf(buf, "%s [%s] new player.", GET_NAME(d->character), d->host);
      mudlog(buf, d->character, LOG_CONNLOG, TRUE);
      break;
    case CCR_TOTEM:
      if ((i = parse_totem(d, arg)) == TOTEM_UNDEFINED) {
        SEND_TO_Q("\r\nThat's not a totem.\r\nTotem: ", d);
        return;
      } else if (i == RETURN_HELP) // for when they use help
        return;
      GET_TOTEM(d->character) = (ubyte) i;
      if (GET_PFILEPOS(d->character) < 0)
        GET_PFILEPOS(d->character) = create_entry(GET_NAME(d->character));
      init_char(d->character);
      d->character->player.host = str_dup(d->host);
      save_char(d->character, NOWHERE);
      Crash_rentsave(d->character, 0);
      write_spells(d->character);
      SEND_TO_Q(motd, d);
      SEND_TO_Q("\r\n\n*** PRESS RETURN: ", d);
      STATE(d) = CON_RMOTD;
      init_create_vars(d);
      sprintf(buf, "%s [%s] new player.", GET_NAME(d->character), d->host);
      mudlog(buf, d->character, LOG_CONNLOG, TRUE);
      break;
    case CCR_PRIORITY:
      switch (LOWER(*arg)) {
        case 'a': case 'b': case 'c': case 'd': case 'e':
          if (d->ccr.pr[(int)(LOWER(*arg)-'a')] == PR_RACE)
            priority_menu(d);
          else {
            d->ccr.temp = (int)(LOWER(*arg)-'a');
            SEND_TO_Q(assign_menu, d);
            SEND_TO_Q("\r\nPriority to assign (c to clear): ", d);
            d->ccr.mode = CCR_ASSIGN;
          }
          break;
        case 'p':
          ok = TRUE;
          for (i = 0; ok && i < 5; i++)
            ok = (d->ccr.pr[i] != PR_NONE);
          if (ok) {
            if ((d->ccr.pr[0] == PR_RACE && d->ccr.pr[1] == PR_MAGIC) ||
                (d->ccr.pr[4] == PR_RACE && d->ccr.pr[0] == PR_MAGIC) ||
                (d->ccr.pr[0] == PR_RACE && d->ccr.pr[2] == PR_MAGIC) ||
                (d->ccr.pr[4] == PR_RACE && d->ccr.pr[1] == PR_MAGIC))
              set_attributes(d->character, 1);
            else set_attributes(d->character, 0);
            SEND_TO_Q(equip_menu, d);
            SEND_TO_Q("\r\nCategory to purchase from: ", d);
            d->ccr.mode = CCR_EQUIP;
          } else priority_menu(d);
          break;
        case '?':
          help_to_buf(buf2, "priorities");
          strcat(buf2, "\r\n Press [return] to continue ");
          SEND_TO_Q(buf2, d);
          d->ccr.temp = CCR_PRIORITY;
          d->ccr.mode = CCR_AWAIT_CR;
          break;
        default:
          priority_menu(d);
      }
      break;
    case CCR_ASSIGN:
      if (!(i = parse_assign(d, arg)))
        SEND_TO_Q("\r\nThat's not a valid priority!\r\nPriority to assign (c to clear): ", d);
      else priority_menu(d);
      break;
    case CCR_EQUIP:
      switch (LOWER(*arg)) {
        case '1':
          SEND_TO_Q(weapons_menu, d);
          d->ccr.mode = CCR_WEAPON;
          complete_menu(d);
          break;
        case '2':
          SEND_TO_Q(armor_menu, d);
          d->ccr.mode = CCR_ARMOR;
          complete_menu(d);
          break;
        case '3':
          SEND_TO_Q(cyberdeck_menu, d);
          d->ccr.mode = CCR_CYBERDECK;
          complete_menu(d);
          break;
        case '4':
          SEND_TO_Q(cyberware_menu, d);
          d->ccr.mode = CCR_CYBERWARE;
          complete_menu(d);
          break;
        case '5':
          GET_NUYEN(d->character) = 0;
          if ((d->ccr.pr[0] == PR_RACE && d->ccr.pr[1] == PR_MAGIC) ||
              (d->ccr.pr[4] == PR_RACE && d->ccr.pr[0] == PR_MAGIC)) {
            SEND_TO_Q(spell_menu, d);
            d->ccr.mode = CCR_SPELLS;
            complete_menu(d);
            return;
          } else if ((d->ccr.pr[0] == PR_RACE && d->ccr.pr[2] == PR_MAGIC) ||
                     (d->ccr.pr[4] == PR_RACE && d->ccr.pr[1] == PR_MAGIC))
            GET_TRADITION(d->character) = TRAD_ADEPT;
          else GET_TRADITION(d->character) = TRAD_MUNDANE;
          if (GET_PFILEPOS(d->character) < 0)
            GET_PFILEPOS(d->character) = create_entry(GET_NAME(d->character));
          init_char(d->character);
          d->character->player.host = str_dup(d->host);
          save_char(d->character, NOWHERE);
          Crash_rentsave(d->character, 0);
          write_spells(d->character);
          SEND_TO_Q(motd, d);
          SEND_TO_Q("\r\n\n*** PRESS RETURN: ", d);
          STATE(d) = CON_RMOTD;
          init_create_vars(d);
          sprintf(buf, "%s [%s] new player.", GET_NAME(d->character), d->host);
          mudlog(buf, d->character, LOG_CONNLOG, TRUE);
          break;
        default:
          SEND_TO_Q("\r\nThat's not a category!\r\nCategory to purchase from (1-5): ", d);
          break;
      }
      break;
    case CCR_WEAPON:
      if (LOWER(*arg) == 'q') {
        SEND_TO_Q(equip_menu, d);
        SEND_TO_Q("\r\nCategory to purchase from: ", d);
        d->ccr.mode = CCR_EQUIP;
        return;
      }
      i = atoi(arg);
      switch (i) {
        case 1:  nuyen = 1000; num = 632; break;
        case 2:  nuyen = 30;   num = 652; break;
        case 3:  nuyen = 500;  num = 633; break;
        case 4:  nuyen = 500;  num = 634; break;
        case 5:  nuyen = 750;  num = 635; break;
        case 6:  nuyen = 350;  num = 618; break;
        case 7:  nuyen = 550;  num = 621; break;
        case 8:  nuyen = 450;  num = 622; break;
        case 9:  nuyen = 300;  num = 623; break;
        case 10: nuyen = 1000; num = 631; break;
        default:
          SEND_TO_Q("That's not an option!\r\n", d);
          complete_menu(d);
          return;
      }
      if (GET_NUYEN(d->character) < nuyen) {
        SEND_TO_Q("You can't afford that!\r\n", d);
        complete_menu(d);
        return;
      }
      if ((rnum = real_object(num)) == -1) {
        SEND_TO_Q("That item is mysteriously unavailable.\r\n", d);
        complete_menu(d);
        return;
      }

// They haven't increased attributes yet.  Let them carry it all.
#if 0
      if ((IS_CARRYING_N(d->character) >= CAN_CARRY_N(d->character)) ||
         (IS_CARRYING_W(d->character) + obj_proto[rnum].obj_flags.weight >=
          CAN_CARRY_N(d->character))) {
        SEND_TO_Q("You can't carry that.", d);
        complete_menu(d);
        return;
      }
#endif

      GET_NUYEN(d->character) -= nuyen;
      obj = read_object(rnum, REAL);
      obj_to_char(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      SEND_TO_Q(weapons_menu, d);
      complete_menu(d);
      break;
    case CCR_ARMOR:
      switch (LOWER(*arg)) {
        case '1': nuyen = 400; num = 2017; break;
        case '2': nuyen = 200; num = 2016; break;
        case '3': nuyen = 150; num = 1914; break;
        case '4': nuyen = 900; num = 1913; break;
        case 'q':
          SEND_TO_Q(equip_menu, d);
          SEND_TO_Q("\r\nCategory to purchase from: ", d);
          d->ccr.mode = CCR_EQUIP;
          return;
        default:
          SEND_TO_Q("That's not an option!\r\n", d);
          complete_menu(d);
          return;
      }
      if (GET_NUYEN(d->character) < nuyen) {
        SEND_TO_Q("You can't afford that!\r\n", d);
        complete_menu(d);
        return;
      }
      if ((rnum = real_object(num)) == -1) {
        SEND_TO_Q("That item is mysteriously unavailable.\r\n", d);
        complete_menu(d);
        return;
      }
#if 0
      if ((IS_CARRYING_N(d->character) >= CAN_CARRY_N(d->character)) ||
         (IS_CARRYING_W(d->character) + obj_proto[rnum].obj_flags.weight >=
          CAN_CARRY_N(d->character))) {
        SEND_TO_Q("You can't carry that.", d);
        complete_menu(d);
        return;
      }
#endif
      GET_NUYEN(d->character) -= nuyen;
      obj = read_object(rnum, REAL);
      obj_to_char(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      SEND_TO_Q(armor_menu, d);
      complete_menu(d);
      break;
    case CCR_CYBERDECK:
      switch (LOWER(*arg)) {
        case '1': nuyen = 6800;  num = 1800; break;
        case '2': nuyen = 12600; num = 1820; break;
        case '3': nuyen = 1200;  num = 1821; break;
        case '4': nuyen = 1200;  num = 1822; break;
        case '5': nuyen = 800;   num = 1823; break;
        case '6': nuyen = 800;   num = 1824; break;
        case '7': nuyen = 800;   num = 1825; break;
        case 'q':
          SEND_TO_Q(equip_menu, d);
          SEND_TO_Q("\r\nCategory to purchase from: ", d);
          d->ccr.mode = CCR_EQUIP;
          return;
        default:
          SEND_TO_Q("That's not an option!\r\n", d);
          complete_menu(d);
          return;
      }
      if (GET_NUYEN(d->character) < nuyen) {
        SEND_TO_Q("You can't afford that!\r\n", d);
        complete_menu(d);
        return;
      }
      if ((rnum = real_object(num)) == -1) {
        SEND_TO_Q("That item is mysteriously unavailable.\r\n", d);
        complete_menu(d);
        return;
      }
#if 0
      if ((IS_CARRYING_N(d->character) >= CAN_CARRY_N(d->character)) ||
         (IS_CARRYING_W(d->character) + obj_proto[rnum].obj_flags.weight >=
          CAN_CARRY_N(d->character))) {
        SEND_TO_Q("You can't carry that.", d);
        complete_menu(d);
        return;
      }
#endif
      GET_NUYEN(d->character) -= nuyen;
      obj = read_object(rnum, REAL);
      obj_to_char(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      SEND_TO_Q(cyberdeck_menu, d);
      complete_menu(d);
      break;
    case CCR_CYBERWARE:
      switch (LOWER(*arg)) {
        case '1': nuyen = 1000;  num = 304; ess = 20;
          for (obj = d->character->cyberware; obj; obj = obj->next_content)
            if (GET_OBJ_VAL(obj, 2) == 1) {
              SEND_TO_Q("You already have that!\r\n", d);
              complete_menu(d);
              return;
            }
          break;
        case '2': nuyen = 4000;  num = 371; ess = 50;
          for (obj = d->character->cyberware; obj; obj = obj->next_content)
            if (GET_OBJ_VAL(obj, 2) == 2) {
              SEND_TO_Q("You already have that!\r\n", d);
              complete_menu(d);
              return;
            } else if (GET_OBJ_VAL(obj, 2) == 3) {
              SEND_TO_Q("You can't have both a radio and a radio receiver!\r\n", d);
              complete_menu(d);
              return;
            }
          break;
        case '3': nuyen = 2000;  num = 372; ess = 25;
          for (obj = d->character->cyberware; obj; obj = obj->next_content)
            if (GET_OBJ_VAL(obj, 2) == 3) {
              SEND_TO_Q("You already have that!\r\n", d);
              complete_menu(d);
              return;
            } else if (GET_OBJ_VAL(obj, 2) == 2) {
              SEND_TO_Q("You can't have both a radio and a radio receiver!\r\n", d);
              complete_menu(d);
              return;
            }
          break;
        case '4': nuyen = 3700;  num = 373; ess = 25;
          for (obj = d->character->cyberware; obj; obj = obj->next_content)
            if (GET_OBJ_VAL(obj, 2) == 4) {
              SEND_TO_Q("You already have that!\r\n", d);
              complete_menu(d);
              return;
            }
          break;
        case '5': nuyen = 9000;  num = 368; ess = 20;
          for (obj = d->character->cyberware; obj; obj = obj->next_content)
            if (GET_OBJ_VAL(obj, 2) == 19) {
              SEND_TO_Q("You can't get two of those!\r\n", d);
              complete_menu(d);
              return;
            }
          break;
        case '6': nuyen = 11500; num = 369; ess = 30;
          for (obj = d->character->cyberware; obj; obj = obj->next_content)
            if (GET_OBJ_VAL(obj, 2) == 19) {
              SEND_TO_Q("You can't get two of those!\r\n", d);
              complete_menu(d);
              return;
            }
          break;
        case 'q':
          SEND_TO_Q(equip_menu, d);
          SEND_TO_Q("\r\nCategory to purchase from: ", d);
          d->ccr.mode = CCR_EQUIP;
          return;
        default:
          SEND_TO_Q("That's not an option!\r\n", d);
          complete_menu(d);
          return;
      }
      if (GET_NUYEN(d->character) < nuyen) {
        SEND_TO_Q("You can't afford that!\r\n", d);
        complete_menu(d);
        return;
      }
      if (d->character->real_abils.ess < ess) {
        SEND_TO_Q("You don't have enough essence to install that.\r\n", d);
        complete_menu(d);
        return;
      }
      if ((rnum = real_object(num)) == -1) {
        SEND_TO_Q("That item is mysteriously unavailable.\r\n", d);
        complete_menu(d);
        return;
      }
      GET_NUYEN(d->character) -= nuyen;
      obj = read_object(rnum, REAL);
      obj_to_cyberware(obj, d->character);
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NODONATE | ITEM_NOSELL);
      SEND_TO_Q(cyberware_menu, d);
      complete_menu(d);
      break;
    case CCR_SPELLS:
      if (LOWER(*arg) == 'q') {
        d->ccr.mode = CCR_TRADITION;
        SEND_TO_Q("\r\nFollow [h]ermetic or [s]hamanic tradition? ", d);
        return;
      } else if (*arg > '7' || *arg < '1') {
        SEND_TO_Q("That's not an option!\r\n", d);
        complete_menu(d);
        return;
      }
      d->ccr.temp = (int)(*arg - '0');
      d->ccr.mode = CCR_FORCE;
      sprintf(buf2, "Force (%d force points remaining): ", d->ccr.force_points);
      SEND_TO_Q(buf2, d);
      break;
    case CCR_FORCE:
      force = atoi(arg);
      switch (d->ccr.temp) {
        case 1:  num = SPELL_MANA_DART;     break;
        case 2:  num = SPELL_MANA_MISSILE;  break;
        case 3:  num = SPELL_MANA_BOLT;     break;
        case 4:  num = SPELL_POWER_DART;    break;
        case 5:  num = SPELL_POWER_MISSILE; break;
        case 6:  num = SPELL_POWER_BOLT;    break;
        case 7:  num = SPELL_HEAL;          break;
        default: num = SPELL_MANA_DART;     break;
      }
      if (force > 4) {
        SEND_TO_Q("You can't learn a spell higher than force 4 for now.\r\n", d);
        sprintf(buf2, "Force (%d force points remaining): ", d->ccr.force_points);
        SEND_TO_Q(buf2, d);
        return;
      }
      if (force < 0) {
        SEND_TO_Q("You can't decrease a spells force below zero.\n\r",d);
	sprintf(buf2,"Force (%d force points remaining): ", d->ccr.force_points);
	SEND_TO_Q(buf2,d);
	return;
      }
      if ((spell = find_spell(d->character, (char *)spells[num]))) {
        if ((force - spell->force) > d->ccr.force_points) {
          SEND_TO_Q("You don't have that many force points to spend!\r\n", d);
          sprintf(buf2, "Force (%d force points remaining): ", d->ccr.force_points);
          SEND_TO_Q(buf2, d);
          return;
        }
        if (!force) {
          for (temp = d->character->spells; temp; temp = temp->next)
            if (temp->next && temp->next == spell)
              break;
          if (temp)
            temp->next = spell->next;
          else d->character->spells = spell->next;
          d->ccr.force_points += spell->force;
          delete spell;
          d->ccr.mode = CCR_SPELLS;
          SEND_TO_Q(spell_menu, d);
          complete_menu(d);
          return;
        }
        d->ccr.force_points += (spell->force - force);
        spell->force = force;
      } else {
        if (force > d->ccr.force_points) {
          SEND_TO_Q("You don't have that many force points to spend!\r\n", d);
          sprintf(buf2, "Force (%d force points remaining): ", d->ccr.force_points);
          SEND_TO_Q(buf2, d);
          return;
        }
        if (!force) {
          d->ccr.mode = CCR_SPELLS;
          SEND_TO_Q(spell_menu, d);
          complete_menu(d);
          return;
        }
        spell = new spell_t;
        i = strlen(spells[num]);
        spell->name = new char[i+1];
        strcpy(spell->name, spells[num]);
        spell->physical = grimoire[num].physical;
        spell->category = grimoire[num].category;
        spell->force = force;
        spell->target = grimoire[num].target;
        spell->drain = grimoire[num].drain;
        spell->damage = grimoire[num].damage;
        spell->type = num;
        spell->effect = SPELL_ELEMENT_NONE;
        if (!d->character->spells) {
          d->character->spells = spell;
          spell->next = NULL;
        } else {
          spell->next = d->character->spells;
          d->character->spells = spell;
        }
        d->ccr.force_points -= force;
      }
      d->ccr.temp = 0;
      d->ccr.mode = CCR_SPELLS;
      SEND_TO_Q(spell_menu, d);
      complete_menu(d);
      break;
    case CCR_TRADITION:
      if (LOWER(*arg) == 'h' || LOWER(*arg) == 's') {
        if (LOWER(*arg) == 'h')
          GET_TRADITION(d->character) = TRAD_HERMETIC;
        else {
          GET_TRADITION(d->character) = TRAD_SHAMANIC;
          SEND_TO_Q(totem_menu, d);
          SEND_TO_Q("\r\nTotem: ", d);
          d->ccr.mode = CCR_TOTEM;
          return;
        }
        if (GET_PFILEPOS(d->character) < 0)
          GET_PFILEPOS(d->character) = create_entry(GET_NAME(d->character));
        init_char(d->character);
        d->character->player.host = str_dup(d->host);
        save_char(d->character, NOWHERE);
        Crash_rentsave(d->character, 0);
        write_spells(d->character);
        SEND_TO_Q(motd, d);
        SEND_TO_Q("\r\n\n*** PRESS RETURN: ", d);
        STATE(d) = CON_RMOTD;
        init_create_vars(d);
        sprintf(buf, "%s [%s] new player.", GET_NAME(d->character), d->host);
        mudlog(buf, d->character, LOG_CONNLOG, TRUE);
      } else SEND_TO_Q("\r\nThat's not a tradition!\r\nFollow [h]ermetic or [s]hamanic tradition? ", d);
      break;
  }
}

int invalid_class(struct char_data *ch, struct obj_data *obj)
{
  int counter;

  if (IS_OBJ_STAT(obj, ITEM_GODONLY) && !IS_NPC(ch) && GET_LEVEL(ch) < LVL_LEGEND)
    return 1;

  if (!IS_NPC(ch) && GET_LEVEL(ch) < LVL_LEGEND)
    if (from_ip_zone(GET_OBJ_VNUM(obj)) || IS_OBJ_STAT(obj, ITEM_VOLATILE))
      return 1;

  return 0;
}
